<?php

namespace Drupal\firebase_ui\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Routing\UrlGeneratorInterface;

/**
 * Controller for displaying a list of anonymous devices.
 */
class AnonymousDeviceListController extends ControllerBase {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The form builder service.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The URL generator service.
   *
   * @var \Drupal\Core\Routing\UrlGeneratorInterface
   */
  protected $urlGenerator;

  /**
   * Constructs a new AnonymousDeviceListController object.
   */
  public function __construct(
    Connection $database,
    FormBuilderInterface $formBuilder,
    DateFormatterInterface $dateFormatter,
    ConfigFactoryInterface $configFactory,
    RequestStack $requestStack,
    UrlGeneratorInterface $urlGenerator,
  ) {
    $this->database = $database;
    $this->formBuilder = $formBuilder;
    $this->dateFormatter = $dateFormatter;
    $this->configFactory = $configFactory;
    $this->requestStack = $requestStack;
    $this->urlGenerator = $urlGenerator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('form_builder'),
      $container->get('date.formatter'),
      $container->get('config.factory'),
      $container->get('request_stack'),
      $container->get('url_generator')
    );
  }

  /**
   * View the list of anonymous Firebase UI devices.
   *
   * @return array
   *   A render array containing the filter form and table.
   */
  public function view() {
    $config = $this->configFactory->get('firebase_ui.settings');

    if (!$config->get('enableForAnonymousUser')) {
      return [
        '#markup' => $this->t('Anonymous device tracking is currently <strong>disabled</strong>. Please enable it from the <a href=":settings">Firebase UI Settings</a>.', [
          ':settings' => $this->urlGenerator->generateFromRoute('firebase_ui.settings'),
        ]),
      ];
    }

    $form = $this->formBuilder->getForm('Drupal\firebase_ui\Form\AnonymousDeviceFilterForm');
    $filters = $this->requestStack->getCurrentRequest()->query->all();

    $header = [
      ['data' => $this->t('Device ID')],
      ['data' => $this->t('OS Type')],
      ['data' => $this->t('Browser')],
      ['data' => $this->t('Device Type')],
      ['data' => $this->t('IP Address')],
      ['data' => $this->t('Created')],
    ];

    $query = $this->database->select('firebase_ui_device', 'd')
      ->fields('d', ['token', 'os_type', 'browser_type', 'device_type', 'ip_address', 'created'])
      ->orderBy('created', 'DESC');

    if (!empty($filters['ip_address'])) {
      $query->condition('ip_address', '%' . $filters['ip_address'] . '%', 'LIKE');
    }
    if (!empty($filters['os_type'])) {
      $query->condition('os_type', '%' . $filters['os_type'] . '%', 'LIKE');
    }
    if (!empty($filters['browser_type'])) {
      $query->condition('browser_type', '%' . $filters['browser_type'] . '%', 'LIKE');
    }
    if (!empty($filters['device_type'])) {
      $query->condition('device_type', '%' . $filters['device_type'] . '%', 'LIKE');
    }

    $result = $query->execute();

    $rows = [];
    foreach ($result as $row) {
      $rows[] = [
        'data' => [
          $row->token,
          $row->os_type,
          $row->browser_type,
          $row->device_type,
          $row->ip_address,
          $this->dateFormatter->format($row->created, 'short'),
        ],
      ];
    }

    return [
      'filter_form' => $form,
      'table' => [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
        '#empty' => $this->t('No anonymous devices found.'),
      ],
    ];
  }

}
