<?php

namespace Drupal\firebase_ui\Controller;

use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\firebase_ui\Service\UserInfoService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\firebase_ui\Entity\FirebaseUiDevice;

/**
 * Controller to handle Firebase UI token saving.
 */
class FirebaseUiController extends ControllerBase {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The user info service.
   *
   * @var \Drupal\firebase_ui\Service\UserInfoService
   */
  protected $userInfoService;

  /**
   * The logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * Constructs a FirebaseUiController object.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\firebase_ui\Service\UserInfoService $user_info_service
   *   The user info service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger service.
   */
  public function __construct(AccountProxyInterface $current_user, EntityTypeManagerInterface $entity_type_manager, UserInfoService $user_info_service, LoggerChannelFactoryInterface $logger_factory) {
    $this->currentUser = $current_user;
    $this->entityTypeManager = $entity_type_manager;
    $this->userInfoService = $user_info_service;
    $this->loggerFactory = $logger_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('current_user'),
      $container->get('entity_type.manager'),
      $container->get('firebase_ui.user_info_service'),
      $container->get('logger.factory')
    );
  }

  /**
   * Saves a Firebase token for the authenticated user.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response indicating success or failure.
   */
  public function saveToken(Request $request): JsonResponse {
    $userInfo = $this->userInfoService->getUserInfo();

    // Get the token from request.
    $data = json_decode($request->getContent(), TRUE);
    $token = $data['token'] ?? NULL;

    if (!$token) {
      return new JsonResponse(['status' => 'error', 'message' => 'No token provided'], 400);
    }

    if ($this->currentUser->isAuthenticated()) {
      /** @var \Drupal\user\UserInterface $account */
      $account = $this->entityTypeManager->getStorage('user')->load($this->currentUser->id());

      $tokens = $account->get('field_firebase_tokens')->getValue();
      $existing_tokens = array_column($tokens, 'value');

      if (!in_array($token, $existing_tokens, TRUE)) {
        $tokens[] = ['value' => $token];
        $account->set('field_firebase_tokens', $tokens);
        $account->save();
      }
    }
    else {
      // dd($token);
      // Anonymous user: store info in custom entity using unique device_id.
      $storage = $this->entityTypeManager->getStorage('firebase_ui_device');
      $existing = $storage->loadByProperties(['token' => $token]);

      if (empty($existing)) {
        $device = FirebaseUiDevice::create([
          'device_id' => $userInfo['device_id'],
          'os_type' => $userInfo['os_type'] ?? '',
          'browser_type' => $userInfo['browser_type'] ?? '',
          'device_type' => $userInfo['device_type'] ?? '',
          'ip_address' => $userInfo['ip_address'] ?? '',
          'token' => $token,
        ]);
        $device->save();
      }
      else {
        // Update existing token for that device ID.
        $device = reset($existing);
        $device->set('token', $token);
        $device->save();
      }
    }

    return new JsonResponse(['status' => 'success']);
  }

}
