<?php

namespace Drupal\firebase_ui\Service;

use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides a service for getting user information.
 */
class UserInfoService {

  /**
   * The request stack service.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Constructs the UserInfoService.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack service.
   */
  public function __construct(RequestStack $request_stack) {
    $this->requestStack = $request_stack;
  }

  /**
   * Get all user information.
   *
   * @return array
   *   An array containing user information.
   */
  public function getUserInfo() {
    // Get Device ID (from cookies or generate a new one).
    $deviceId = $this->getDeviceId();

    // Get OS Type (from User-Agent).
    $osType = $this->getOsType();

    // Get Browser Type (from User-Agent).
    $browserType = $this->getBrowserType();

    // Get Device Type (Mobile/Desktop).
    $deviceType = $this->getDeviceType();

    // Get IP Address.
    $ipAddress = $this->getIpAddress();

    // Return all collected data.
    return [
      'device_id' => $deviceId,
      'os_type' => $osType,
      'browser_type' => $browserType,
      'device_type' => $deviceType,
      'ip_address' => $ipAddress,
    ];
  }

  /**
   * Get Device ID (from cookies or create a new one).
   *
   * @return string
   *   The device ID.
   */
  private function getDeviceId() {
    $deviceId = $this->requestStack->getCurrentRequest()->cookies->get('device_id');

    if (!$deviceId) {
      // Generate a unique ID (UUID).
      $deviceId = bin2hex(random_bytes(16));

      // Set the cookie with a 1-year expiration.
      setcookie('device_id', $deviceId, time() + (86400 * 365), '/');
    }

    return $deviceId;
  }

  /**
   * Get the operating system type.
   *
   * @return string
   *   The operating system type.
   */
  private function getOsType() {
    $userAgent = $this->requestStack->getCurrentRequest()->headers->get('User-Agent');

    if (preg_match('/windows/i', $userAgent)) {
      return 'Windows';
    }
    elseif (preg_match('/macintosh|mac os x/i', $userAgent)) {
      return 'macOS';
    }
    elseif (preg_match('/linux/i', $userAgent)) {
      return 'Linux';
    }
    elseif (preg_match('/iphone|ipad/i', $userAgent)) {
      return 'iOS';
    }
    elseif (preg_match('/android/i', $userAgent)) {
      return 'Android';
    }

    return 'Unknown';
  }

  /**
   * Get the browser type.
   *
   * @return string
   *   The browser type.
   */
  private function getBrowserType() {
    $userAgent = $this->requestStack->getCurrentRequest()->headers->get('User-Agent');

    if (strpos($userAgent, 'Firefox') !== FALSE) {
      return 'Firefox';
    }
    if (strpos($userAgent, 'Chrome') !== FALSE) {
      return 'Chrome';
    }
    if (strpos($userAgent, 'Safari') !== FALSE) {
      return 'Safari';
    }
    if (strpos($userAgent, 'Edge') !== FALSE) {
      return 'Edge';
    }

    return 'Unknown';
  }

  /**
   * Get the device type (Desktop or Mobile).
   *
   * @return string
   *   The device type.
   */
  private function getDeviceType() {
    $userAgent = $this->requestStack->getCurrentRequest()->headers->get('User-Agent');

    if (preg_match('/mobile/i', $userAgent)) {
      return 'Mobile';
    }

    return 'Desktop';
  }

  /**
   * Get the user's IP address.
   *
   * @return string
   *   The IP address.
   */
  private function getIpAddress() {
    return $this->requestStack->getCurrentRequest()->getClientIp();
  }

}
